import { useDevice } from 'chayns-api';
import { format } from 'prettier/standalone';
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { PrismAsyncLight as SyntaxHighlighter } from 'react-syntax-highlighter';
import { oneDark, oneLight } from 'react-syntax-highlighter/dist/esm/styles/prism';
import { CodeHighlighterTheme } from '../../types/codeHighlighter';
import { formatLanguage, getParserForLanguage } from '../../utils/codeHighlighter';
import { StyledCodeHighlighter, StyledCodeHighlighterFileName, StyledCodeHighlighterHeader } from './CodeHighlighter.styles';
import CopyToClipboard from './copy-to-clipboard/CopyToClipboard';
const CodeHighlighter = _ref => {
  let {
    theme = CodeHighlighterTheme.Dark,
    code,
    copyButtonText,
    language,
    highlightedLines,
    shouldFormatCode = false,
    onFormatError,
    shouldShowLineNumbers = false
  } = _ref;
  const [width, setWidth] = useState(0);
  const ref = useRef(null);
  const {
    browser
  } = useDevice();
  useEffect(() => {
    if (ref.current) {
      const {
        children
      } = ref.current;
      const preElement = Array.from(children).find(_ref2 => {
        let {
          tagName
        } = _ref2;
        return tagName.toLowerCase() === 'pre';
      });
      if (preElement) {
        setWidth(preElement.scrollWidth);
      }
    }
  }, []);

  // function to style highlighted code
  const lineWrapper = useCallback(lineNumber => {
    let style = {
      backgroundColor: 'none',
      display: 'block',
      borderRadius: '2px',
      width: width - 15
    };
    if (highlightedLines?.added && highlightedLines.added.includes(lineNumber)) {
      style = {
        ...style,
        backgroundColor: '#2EF29930'
      };
    } else if (highlightedLines?.removed && highlightedLines.removed.includes(lineNumber)) {
      style = {
        ...style,
        backgroundColor: '#F22E5B30'
      };
    } else if (highlightedLines?.marked && highlightedLines.marked.includes(lineNumber)) {
      style = {
        ...style,
        backgroundColor: '#cccccc40'
      };
    }
    return {
      style
    };
  }, [highlightedLines, width]);
  const formattedCode = useMemo(() => {
    if (language) {
      const config = getParserForLanguage(language);
      if (shouldFormatCode && config) {
        try {
          return format(code, config);
        } catch (error) {
          if (typeof onFormatError !== 'undefined') onFormatError(error);
        }
      }
      return code;
    }
    return code;
  }, [code, language, shouldFormatCode, onFormatError]);
  useEffect(() => {
    const elements = document.getElementsByClassName('linenumber');
    Array.from(elements).forEach(element => {
      const wrapper = document.createElement('twIgnore');
      while (element.firstChild) {
        wrapper.appendChild(element.firstChild);
      }
      element.appendChild(wrapper);
    });
  }, []);
  return useMemo(() => /*#__PURE__*/React.createElement(StyledCodeHighlighter, {
    $browser: browser?.name,
    $codeTheme: theme,
    ref: ref
  }, /*#__PURE__*/React.createElement(StyledCodeHighlighterHeader, {
    $codeTheme: theme
  }, /*#__PURE__*/React.createElement(StyledCodeHighlighterFileName, {
    $codeTheme: theme
  }, /*#__PURE__*/React.createElement("twIgnore", null, formatLanguage(language))), /*#__PURE__*/React.createElement(CopyToClipboard, {
    text: code,
    theme: theme,
    copyButtonText: copyButtonText
  })), /*#__PURE__*/React.createElement(SyntaxHighlighter, {
    language: language ?? '',
    showLineNumbers: shouldShowLineNumbers,
    style: theme === CodeHighlighterTheme.Dark ? oneDark : oneLight,
    wrapLines: true,
    lineProps: lineWrapper
  }, formattedCode)), [browser?.name, theme, language, code, copyButtonText, shouldShowLineNumbers, lineWrapper, formattedCode]);
};
CodeHighlighter.displayName = 'CodeHighlighter';
export default CodeHighlighter;
//# sourceMappingURL=CodeHighlighter.js.map